import { DECISION_MAKER } from '@modules/authorization/authorization.di-tokens';
import { DecisionQuery } from '@modules/authorization/core/application/queries/decision/decision.query';
import { DecisionQueryHandler } from '@modules/authorization/core/application/queries/decision/decision.query-handler';
import {
  Action,
  Domain,
} from '@modules/authorization/core/domain/authorization.types';

import { Test, TestingModule } from '@nestjs/testing';

const mockDecisionMaker = {
  decide: jest
    .fn()
    .mockImplementationOnce(() => true)
    .mockImplementationOnce(() => false),
};

describe('Decision Query Handler', () => {
  let decisionQueryHandler: DecisionQueryHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: DECISION_MAKER,
          useValue: mockDecisionMaker,
        },
        DecisionQueryHandler,
      ],
    }).compile();

    decisionQueryHandler =
      module.get<DecisionQueryHandler>(DecisionQueryHandler);
  });

  it('should be defined', () => {
    expect(decisionQueryHandler).toBeDefined();
  });

  describe('execution', () => {
    it('should return a positive decision', async () => {
      const decisionQuery = new DecisionQuery(Domain.USER, Action.READ, [
        {
          name: 'owner',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a8',
        },
        {
          name: 'id',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a8',
        },
      ]);
      const decision: boolean = await decisionQueryHandler.execute(
        decisionQuery,
      );
      expect(decision).toBeTruthy();
    });
    it('should return a negative decision', async () => {
      const decisionQuery = new DecisionQuery(Domain.USER, Action.READ, [
        {
          name: 'owner',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a8',
        },
        {
          name: 'id',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a9',
        },
      ]);
      const decision: boolean = await decisionQueryHandler.execute(
        decisionQuery,
      );
      expect(decision).toBeFalsy();
    });
  });
});
